#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 emitter = 1; /* SMU1 */
/*ViInt32 base; open */
ViInt32 collector = 4; /* SMU4 */
ViReal64 start = 0;
ViReal64 vc = 100;
ViReal64 iccomp = 0.005;
ViReal64 hold = 0;
ViReal64 delay = 0;
ViReal64 meas;
ViInt32 status; /* 13 */
ret = age5270_setSwitch(vi, emitter, 1); /* 15 */
ret = age5270_setSwitch(vi, collector, 1);
check_err (vi, ret); /* 17 */
ret = age5270_force(vi, emitter, age5270_VF_MODE, 0, 0, 0.1, 0);
check_err (vi, ret); /* 20 */
ret = age5270_setBdv(vi, collector, 0, start, vc, iccomp, hold, delay);
check_err (vi, ret); /* 23 */
ret = age5270_measureBdv(vi, age5270_SHORT_INTERVAL, &meas, &status); /* 25 */
check_err (vi, ret);
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 28 */
check_err (vi, ret);
if (status = 8){ /* 31 */
printf("Vbd = %8.6f V \n", meas);
}
else {
printf("Error occurred during measurement.\n");
printf("Status code = %d \n", status);
} /* 37 */
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0); /* 39 */
check_err (vi, ret);
}
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 ret = age5270_cmd(vi, "aa");
 check_err(vi, ret);
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}