#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 drain = 1; /* SMU1, drain */
ViInt32 gate = 2; /* SMU2, gate */
ViInt32 source = 4; /* SMU4, source */
ViInt32 bulk = 6; /* SMU6, bulk */
ViReal64 vd = 5;
ViReal64 vg = 0;
ViReal64 idcomp = 0.05;
ViReal64 igcomp = 0.01;
ViReal64 start = -5;
ViReal64 hold = 0.1;
ViReal64 delay = 0.001;
ViReal64 meas;
ViInt32 status; /* 16 */
ret = age5270_setSwitch(vi, drain, 1); /* 18 */
ret = age5270_setSwitch(vi, gate, 1);
ret = age5270_setSwitch(vi, source, 1);
ret = age5270_setSwitch(vi, bulk, 1);
check_err (vi, ret); /* 22 */
ret = age5270_force(vi, bulk, age5270_VF_MODE, 0, 0, 0.1, 0);
ret = age5270_force(vi, source, age5270_VF_MODE, 0, 0, 0.1, 0);
ret = age5270_force(vi, gate, age5270_VF_MODE, 0, vg, igcomp, 0);
check_err (vi, ret); /* 27 */
ret = age5270_setIleak(vi, drain, 0, vd, idcomp, start, hold, delay); /* 29 */
check_err (vi, ret);
ret = age5270_measureIleak(vi, drain, age5270_SHORT_INTERVAL, &meas, &status); /* 32 */
check_err (vi, ret);
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 35 */
check_err (vi, ret);
printf("Id = %8.6f mA\n", meas * 1000); /* 38 */
printf("Vd = %3.2f to %3.2f V\n", start, vd);
printf("Vg = %3.2f V\n", vg);
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0); /* 42 */
check_err (vi, ret);
}
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 ret = age5270_cmd(vi, "aa");
 check_err(vi, ret);
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}