#include   <stdio.h>
#include   <stdlib.h>
#include   <visa.h>
#include   "age5270.h"
 
void check_err (ViSession vi, ViStatus ret) {
 ViInt32    	inst_err;
 ViChar		    err_msg[256];
 if(VI_SUCCESS > ret) {
     if ( age5270_INSTR_ERROR_DETECTED == ret ) {
         age5270_error_query(vi, &inst_err, err_msg);
         printf("Instrument Error: %ld\n %s\n", inst_err, err_msg);
	 }
     else {
         age5270_error_message(vi, ret, err_msg);
         printf("Driver Error: %ld\n %s\n", ret, err_msg);
	 }
 }
}

void perform_meas (ViSession vi, ViStatus ret) /* 1 */
{
ViInt32 emitter = 1; /* SMU1 */
ViInt32 base = 2; /* SMU2 */
ViInt32 collector = 4; /* SMU4 */
ViReal64 vc = 3;
ViReal64 ib = 150E-6;
ViReal64 iccomp = 0.05;
ViReal64 vbcomp = 5;
ViReal64 hold = 0.1;
ViReal64 width = 0.001;
ViReal64 period = 0.01;
ViInt32 nop1 = 11;
ViInt32 nop2 = 3;
ViInt32 rep;
ViReal64 sc[33];
ViReal64 md[33];
ViInt32 st[33];
ViReal64 tm[33];
ViReal64 dib[3];
ViInt32 i = 0;
ViInt32 j;
ViInt32 n;
ViInt32 smpl = 5;
ViChar f_name[] = "data1.txt";
ViChar head1[] = "Ib (uA), Vc (V), Ic (mA), Time (sec), Status";
ViChar msg1[] = "Saving data...";
ViChar msg2[] = "Data save completed.";
ViChar c = '\n';
ret = age5270_setSwitch(vi, emitter, 1); /* 31 */
ret = age5270_setSwitch(vi, base, 1);
ret = age5270_setSwitch(vi, collector, 1);
check_err (vi, ret); /* 34 */
ret = age5270_setAdc(vi, age5270_HSPEED_ADC, age5270_INTEG_MANUAL, smpl, age5270_FLAG_OFF); /* 36 */
ret = age5270_setAdcType(vi, age5270_CH_ALL, age5270_HSPEED_ADC);
ret = age5270_resetTimestamp(vi);
ret = age5270_force(vi, emitter, age5270_VF_MODE, 0, 0, 0.1, 0);
for (j = 0; j < nop2; j++){ /* 41 */
dib[j] = (j + 1) * ib / nop2;
ret = age5270_force(vi, base, age5270_IF_MODE, 0, dib[j], vbcomp, 0);
ret = age5270_setPiv(vi, collector, age5270_SWP_VF_SGLLIN, 0, base, 0, vc, nop1, hold, width, period, iccomp);
check_err (vi, ret);
ret = age5270_sweepPiv(vi, collector, age5270_IM_MODE, 0, &rep, &sc[i], &md[i], &st[i], &tm[i]);
check_err (vi, ret);
if ( rep = nop1 ) {
i = i + nop1;
}
else {
printf ("%d measurement steps were returned.\nIt must be %d steps.\n", rep, nop1);
ret = age5270_zeroOutput(vi, age5270_CH_ALL);
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0);
check_err (vi, ret);
exit (ret);
}
} /* 60 */
ret = age5270_zeroOutput(vi, age5270_CH_ALL); /* 62 */
check_err (vi, ret);
printf(" Ib (uA), Vc (V), Ic (mA)\n"); /* 64 */
for (j = 0; j < nop2; j++){
n = j * nop1;
for (i = n; i < n + nop1; i++){
printf(" %4.1f, %3.2f, %8.6f \n", dib[j] * 1E6, sc[i], md[i] * 1000);
}
} /* 71 */
FILE *stream; /* 73 */
if( ( stream = fopen( f_name, "w+" )) == NULL ){
printf( "Data file was not opened\n" );
}
else {
printf( "%s%c", msg1, c );
fprintf( stream, "%s%c", head1, c );
for (j = 0; j < nop2; j++){
n = j * nop1;
for (i = n; i < n + nop1; i++){
fprintf( stream, "%4.1f, %3.2f, %8.6f, %8.6f, %d\n", dib[j] * 1E6, sc[i], md[i] * 1000, tm[i], st[i]);
}
}
printf( "%s%c", msg2, c );
}
if( fclose( stream ) ){
printf( "Data file was not closed\n" );
} /* 92 */
ret = age5270_setSwitch(vi, age5270_CH_ALL, 0); /* 94 */
check_err (vi, ret);
}
 
ViStatus main ( )
{
 ViStatus    ret;
 ViSession   vi;
 ViChar      err_msg[256];

 /* Starting the session. */
 ret = age5270_init("GPIB::17::INSTR", VI_TRUE, VI_TRUE, &vi);
 if ( ( ret < VI_SUCCESS ) || ( vi == VI_NULL ) ) {
     printf("Initialization failure.\n Status code: %d.\n", ret);
     if ( vi != VI_NULL ) {
         age5270_error_message(vi, ret, err_msg);
         printf("Error: %ld\n %s\n", ret, err_msg);
	 }
     exit (ret);
 }
 ret = age5270_reset(vi);              	       /* Resets E5270              */
 ret = age5270_timeOut(vi, 60000); 	           /* sets 60 second timeout    */
 ret = age5270_errorQueryDetect(vi, VI_TRUE);  /* Turns on error detection. */
 
 perform_meas (vi, ret);
 ret = age5270_cmd(vi, "aa");
 check_err(vi, ret);
 
 /* Closing the session. */
 ret = age5270_close(vi);
 check_err(vi, ret);

 return VI_SUCCESS;
}