Imports Agilent.TMFramework
Imports Agilent.TMFramework.DataAnalysis
Imports Agilent.TMFramework.DataVisualization
Imports Agilent.TMFramework.InstrumentIO
Imports Agilent.TMFramework.InstrumentDriverInterop
Imports Agilent.TMFramework.InstrumentDriverInterop.Design
Imports Agilent.TMFramework.InstrumentDriverInterop.VxipnpWrappers

Module Module1

    Sub Main()
        Dim Age527x As Age5270 = New Age5270("GPIB0::23::INSTR", True, True)
        Age527x.Reset()
        Age527x.TimeOut(60000)

        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))
        perform_meas(Age527x)

        Age527x.SetSwitch(0, 0)
        Age527x.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub
    Sub perform_meas(ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        Dim nop1 As Integer = 11
        Dim nop2 As Integer = 3
        Dim data(nop2, nop1) As String
        Dim val As String = "Vg (V), Vd (V), Id (mA), Time, Status"
        Dim fname As String = "C:\Agilent\data\ex4.txt"
        Dim title As String = "Sweep Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim t() As Integer = {2, 5, 6, 7}
        Age527x.SetSwitch(t(3), 1)           'SMU4: substrate
        Age527x.SetSwitch(t(2), 1)           'SMU3: source
        Age527x.SetSwitch(t(1), 1)           'SMU2: gate
        Age527x.SetSwitch(t(0), 1)           'SMU1: drain

        Dim vd1 As Double = 0
        Dim vd2 As Double = 3
        Dim idcomp As Double = 0.05
        Dim vg1 As Double = 1
        Dim vg2 As Double = 3
        Dim igcomp As Double = 0.01
        Dim vg As Double = vg1              'secondary sweep output value
        Dim d_vg As Double = 0              'secondary sweep step value (delta)
        If nop2 <> 1 Then d_vg = (vg2 - vg1) / (nop2 - 1)

        Dim hold As Double = 0
        Dim delay As Double = 0
        Dim s_delay As Double = 0
        Dim p_comp As Double = 0

        Dim rep As Integer = nop1
        Dim sc(nop1) As Double
        Dim md(nop1) As Double
        Dim st(nop1) As Integer
        Dim tm(nop1) As Double

        Age527x.Force(t(3), Age5270.ModeEnum1.VoltageOutput, 0, 0, 0.05, 0)
        Age527x.Force(t(2), Age5270.ModeEnum1.VoltageOutput, 0, 0, 0.05, 0)

        For j = 0 To nop2 - 1
            Age527x.SetIv(t(0), Age5270.ModeEnum2.SingleLinearV, 0, vd1, vd2, nop1, hold, delay, s_delay, idcomp, p_comp)
            Age527x.ErrorQuery(err, msg)
            If err <> 0 Then Age527x.ZeroOutput(0) : GoTo Check_err

            Age527x.Force(t(1), Age5270.ModeEnum1.VoltageOutput, 0, vg, igcomp, 0)
            Age527x.ResetTimestamp()                'resets timer
            Age527x.SweepIv(t(0), Age5270.ModeEnum7.CurrentMeasurement, 0, rep, sc, md, st, tm)
            If rep <> nop1 Then Age527x.ZeroOutput(0) : GoTo Check_err

            For i = 0 To nop1 - 1
                data(j, i) = Chr(13) & Chr(10) & vg & ", " & sc(i) & ", " & md(i) * 1000 & ", " & tm(i) & ", " & st(i)
            Next

            vg = vg + d_vg
        Next j

        Age527x.ZeroOutput(0)
        save_data(fname, title, val, data, nop1, nop2, Age527x)

Check_err:
        If err <> 0 Then MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
        If rep <> nop1 Then MsgBox("No. of data: " & rep & " (not " & nop1 & ")", vbOKOnly, "")
    End Sub


    Sub save_data(ByVal fname As String, ByVal title As String, ByVal val As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal Age527x As Age5270)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, val)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                val = val & data(j, i)
            Next
        Next
        val = val & Chr(10) & Chr(10) & "Data save completed."
        val = val & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(val, vbYesNo, title)
        If rbx = vbYes Then perform_meas(Age527x)
    End Sub
End Module
