Imports Ivi.visa.interop

Module Module1


    Sub Main()
        Dim B1500 As IResourceManager
        Dim session As IMessage
        B1500 = New ResourceManager
        session = B1500.Open("GPIB0::17::INSTR")
        session.WriteString("*RST" & vbLf)
        MsgBox("Click OK to start measurement.", vbOKOnly, "")
        Console.WriteLine("Measurement in progress. . ." & Chr(10))

        Dim t() As Integer = {4, 3, 2, 1}
        Dim term As String = t(0) & "," & t(1) & "," & t(2) & "," & t(3)
        session.WriteString("CN " & term & vbLf)
        perform_meas(session, t)

        session.WriteString("CL" & vbLf)
        session.Close()
        MsgBox("Click OK to stop the program.", vbOKOnly, "")
        Console.WriteLine("Measurement completed." & Chr(10))
    End Sub

    Sub perform_meas(ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0                         't(0): Emitter
        Dim j As Integer = 0                         't(1): Base
        Dim nop1 As Integer = 11                     't(2): Collector
        Dim nop2 As Integer = 3                      't(3): not use
        Dim data(nop2 - 1, nop1 - 1) As String
        Dim value As String = "Ib (mA), Vc (V), Ic (mA), Time (sec), Status"
        Dim fname As String = "C:\Agilent\prog_ex\data6.txt"
        Dim title As String = "Measurement Result"
        Dim msg As String = "No error."
        Dim err As Integer = 0

        Dim v0 As Double = 0
        Dim vc1 As Double = 0
        Dim vc2 As Double = 5
        Dim iccomp As Double = 0.05
        Dim ib1 As Double = 0.003
        Dim ib2 As Double = 0.007
        Dim vbcomp As Double = 5
        Dim ib As Double = ib1              'secondary sweep output value
        Dim d_ib As Double = 0              'secondary sweep step value (delta)
        If nop2 <> 1 Then d_ib = (ib2 - ib1) / (nop2 - 1)

        Dim hold As Double = 0
        Dim delay As Double = 0
        Dim s_delay As Double = 0
        Dim rep As Integer = nop1

        Dim mret As String
        Dim sc(nop1) As Double
        Dim md(nop1) As Double
        Dim st(nop1) As String
        Dim tm(nop1) As Double
        session.WriteString("FMT 1,1" & vbLf)   'ASCII, <CRLF EOI>, w/sweep source data
        session.WriteString("TSC 1" & vbLf)     'enables time stamp output
        session.WriteString("FL 1" & vbLf)      'sets filter on
        session.WriteString("AV 10,1" & vbLf)   'sets number of samples for 1 data
        session.WriteString("CL " & t(3) & vbLf)
        session.WriteString("DV " & t(0) & ",0,0,0.1" & vbLf)   'out= 0 V, comp= 0.1 A
        Dim b_pt As String = "0.1,0.01,0.02"       'hold, width, period in sec
        session.WriteString("PT " & b_pt & vbLf)
        session.WriteString("MM 4," & t(2) & vbLf)              '4: pulsed sweep measurement
        session.WriteString("CMM " & t(2) & ",1" & vbLf)        '1: current measurement
        session.WriteString("RI " & t(2) & ",0" & vbLf)         '0: auto ranging
        session.WriteString("WT " & hold & "," & delay & "," & s_delay & vbLf)
        session.WriteString("WM 2,1" & vbLf)                    'stops any abnormal
        session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
        If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err

        For j = 0 To nop2 - 1
            session.WriteString("PWV " & t(2) & ",1,0," & v0 & "," & vc1 & "," & vc2 & "," & nop1 & "," & iccomp & vbLf)
            session.WriteString("DI " & t(1) & ",0," & ib & "," & vbcomp & vbLf)
            session.WriteString("TSR" & vbLf)
            session.WriteString("XE" & vbLf)
            session.WriteString("*OPC?" & vbLf) : rep = session.ReadString(1 + 2)
            session.WriteString("ERR? 1" & vbLf) : err = session.ReadString(4 + 2)
            If err <> 0 Then session.WriteString("DZ" & vbLf) : GoTo Check_err
            session.WriteString("NUB?" & vbLf) : rep = session.ReadString(3 + 2)
            If rep <> nop1 * 3 Then session.WriteString("DZ" & vbLf) : GoTo Check_nop

            mret = session.ReadString(16 * 3 * nop1 + 1)
            For i = 0 To nop1 - 1
                tm(i) = Val(Mid(mret, 4 + 16 * 3 * i, 12))
                st(i) = Mid(mret, 17 + 16 * 3 * i, 3)
                md(i) = Val(Mid(mret, 20 + 16 * 3 * i, 12))
                sc(i) = Val(Mid(mret, 36 + 16 * 3 * i, 12))
                data(j, i) = Chr(13) & Chr(10) & ib * 1000 & ", " & sc(i) & ", " & md(i) * 1000 & ", " & tm(i) & ", " & st(i)
            Next
            ib = ib + d_ib
        Next j
        session.WriteString("DZ" & vbLf)
        save_data(fname, title, value, data, nop1, nop2, session, t)
        Exit Sub

Check_err:
        session.WriteString("EMG? " & err & vbLf) : msg = session.ReadString(256)
        MsgBox("Instrument error: " & err & Chr(10) & msg, vbOKOnly, "")
        Exit Sub

Check_nop:
        MsgBox("No. of data: " & rep & " (not " & nop1 * 3 & ")", vbOKOnly, "")
    End Sub

    Sub save_data(ByVal fname As String, ByVal title As String, ByVal value As String, ByVal data(,) As String, ByVal nop1 As Integer, ByVal nop2 As Integer, ByVal session As IMessage, ByVal t() As Integer)
        Dim i As Integer = 0
        Dim j As Integer = 0
        FileOpen(1, fname, OpenMode.Output, OpenAccess.Write, OpenShare.LockReadWrite)
        Print(1, value)
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                Print(1, data(j, i))
            Next
        Next
        FileClose(1)

        Dim rbx As Integer
        For j = 0 To nop2 - 1
            For i = 0 To nop1 - 1
                value = value & data(j, i)
            Next
        Next
        value = value & Chr(10) & Chr(10) & "Data save completed."
        value = value & Chr(10) & Chr(10) & "Do you want to perform measurement again?"
        rbx = MsgBox(value, vbYesNo, title)
        If rbx = vbYes Then perform_meas(session, t)
    End Sub

End Module
